﻿function Get-InstalledPrograms([string]$include = "*", [switch] $detailed=$false)
{
#EX:
#	Get-InstalledPrograms *desktop* |
#		foreach {(get-itemproperty -path $_.PSPath -name UninstallString -ea SilentlyContinue).UninstallString}


	$name = "";
	$items = dir -path Registry::HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall
	$items += dir -path Registry::HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall
	$items |
		foreach {
			$regObj = $_; 
			if(-not $regObj) { continue; }
			
			$name = (get-itemproperty -path $_.PSPath -name DisplayName -ea SilentlyContinue)
			if(-not $name) {
				$name = $regObj.PSChildName
			}
			else { $name = $name.DisplayName }
			if($detailed) {
				foreach ($vname in $regObj.GetValueNames()) {
					$val = $regObj.GetValue($vname);
					#$regObj[$vname] = $val;
					write-host "$vname=$val"
				}
			}

			echo $regObj
		} |
		where { 
			$_ -like $include -or $name -like $include -or $_.Name -like $include -or $_.PSChildName -like $include
		} | 
		foreach { 

			$_; 
			if($_.PSPath -and $detailed) {
				foreach ($vname in $_.GetValueNames()) {
					$val = $_.GetValue($vname);
					
					write-host "$vname=$val"
				}
				write-host -for yellow (get-itemproperty -path $_.PSPath -name UninstallString -ea SilentlyContinue) -ea SilentlyContinue;
				write-host -for yellow (get-itemproperty -path $_.PSPath -name DisplayName -ea SilentlyContinue) -ea SilentlyContinue;
			}
			
		}
}


function Uninstall-Conditionally(
		[Parameter(Mandatory=$true)]
		[string]$installerKeyName, 
		[Parameter(Mandatory=$true)]
		[string]$newDisplayVersion, 
		[Parameter(Mandatory=$true)]
		[int]$newVersion, 
		[Parameter(Mandatory=$true)]
		[string]$packageName,
		[Parameter(Mandatory=$true)]
		[string]$logDir) {
	#check for existing PowerGUI installs
	$powerguiRegs = get-installedprograms "$installerKeyName"
	
	foreach($powerguiReg in $powerguiRegs) {

		$installedDisplayVersion = $powerguiReg.GetValue("DisplayVersion");
		$installedVersion = $powerguiReg.GetValue("Version");
		
		if($newDisplayVersion -like $installedDisplayVersion) {
			Write-Warning "$packageName $installedDisplayVersion is already installed (but not by Chocolatey!), not installing new version"
			return $false;
		}
		elseif($installedVersion -lt $newversion) {
			Write-Warning "$packageName $installedDisplayVersion is lower than new version: $newDisplayVersion, will uninstall existing version"
			$msiUninstallLoggingArgs = "/lvoicewarmupx `"$logDir\${packageName}${installedDisplayVersion}_uninstall.log`""
			Start-ChocolateyProcessAsAdmin "/quiet $msiUninstallLoggingArgs /x$installerKeyName" 'msiexec'
			return $true

		} else {
			throw "$packageName $installedDisplayVersion is a higher version than the version attempting to be installed ($newDisplayVersion), will not install"
		}
		
	}

}

Export-ModuleMember Get-InstalledPrograms
Export-ModuleMember Uninstall-Conditionally
